# ----------------------------------------
#
# Thank you, Heroku gem, for nearly all of this!
#
# ----------------------------------------


def generate_ssh_key(keyfile)
  ssh_dir = File.join(home_directory, ".ssh")
  unless File.exists?(ssh_dir)
    FileUtils.mkdir_p ssh_dir
    File.chmod(0700, ssh_dir)
  end
  `ssh-keygen -t rsa -N "" -f \"#{home_directory}}/.ssh/#{keyfile}\" 2>&1`
end

def get_key(key)
  File.read(key).chomp #strip trailing newline if present
end

def ask_user
  $stdin.gets.to_s.strip
end

def error(message)
  $stderr.puts(message)
  exit(1)
end

def home_directory
  running_on_windows? ? ENV['USERPROFILE'].gsub("\\","/") : ENV['HOME']
end

def running_on_windows?
  RUBY_PLATFORM =~ /mswin32|mingw32/
end

def get_or_generate_ssh_key
  public_keys = Dir.glob("#{home_directory}/.ssh/*.pub").sort

  case public_keys.length
  when 0 then
    puts "Could not find an existing public key."
    puts "Let's generate one..."
    puts "Generating new SSH public key."
    generate_ssh_key("id_rsa")
    get_key("#{home_directory}/.ssh/id_rsa.pub")
  when 1 then
    puts "Using existing public key: #{public_keys.first}"
    get_key(public_keys.first)
  else
    puts "Found the following SSH public keys:"
    public_keys.each_with_index do |key, index|
      puts "#{index+1}) #{File.basename(key)}"
    end
    puts "Which would you like to use? (Enter number) "
    choice = ask_user.to_i - 1
    chosen = public_keys[choice]
    if choice == -1 || chosen.nil?
      error("Invalid choice.")
    end
    get_key(chosen)
  end
end
